#include <WiFi.h>
#include <ESP32Servo.h>

// WiFi credentials
const char* ssid = "EngineeringStudent";
const char* password = "cls2024!";

// WiFi server
WiFiServer server(80);

// Define servo objects
Servo servo1, servo2, servo3, servo4;

// Updated servo pins
const int servoPins[] = {4, 5, 7, 21};

void runServoSequence() {
  Serial.println("Running servo sequence...");

  for (int angle = 0; angle <= 180; angle += 30) {
    servo1.write(angle);
    servo2.write(angle);
    servo3.write(angle);
    servo4.write(angle);
    delay(300);
  }

  for (int angle = 180; angle >= 0; angle -= 30) {
    servo1.write(angle);
    servo2.write(angle);
    servo3.write(angle);
    servo4.write(angle);
    delay(300);
  }

  Serial.println("Sequence complete.");
}

void setup() {
  Serial.begin(115200);

  // Attach servos to new pins
  servo1.attach(servoPins[0]);
  servo2.attach(servoPins[1]);
  servo3.attach(servoPins[2]);
  servo4.attach(servoPins[3]);

  // Set initial positions
  servo1.write(90);
  servo2.write(90);
  servo3.write(90);
  servo4.write(90);

  // Connect to WiFi
  WiFi.begin(ssid, password);
  Serial.print("Connecting to WiFi");
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nConnected to WiFi");
  Serial.print("IP Address: ");
  Serial.println(WiFi.localIP());

  // Start server
  server.begin();
  Serial.println("Server started, waiting for connections...");
}

void loop() {
  WiFiClient client = server.available();

  if (client) {
    Serial.println("New Client Connected!");
    String request = "";

    while (client.connected()) {
      if (client.available()) {
        char c = client.read();
        request += c;

        if (request.endsWith("\r\n\r\n")) {
          break;
        }
      }
    }

    if (request.indexOf("GET /run") >= 0) {
      runServoSequence();
      client.println("HTTP/1.1 200 OK");
      client.println("Content-Type: text/plain");
      client.println("Connection: close");
      client.println();
      client.println("Servo sequence started.");
    } else {
      client.println("HTTP/1.1 200 OK");
      client.println("Content-Type: text/html");
      client.println("Connection: close");
      client.println();
      client.println("<html><body>");
      client.println("<h1>ESP32 Servo Control</h1>");
      client.println("<p><a href=\"/run\">Run Servo Sequence</a></p>");
      client.println("</body></html>");
    }

    delay(10);
    client.stop();
    Serial.println("Client Disconnected.");
  }
}

